@@ -53,6 +53,7 @@ For example::
 from __future__ import division
 
 from abc import ABCMeta, abstractproperty, abstractmethod
+import __builtin__
 import collections
 import itertools
 import threading
@@ -62,7 +63,15 @@ import numpy as np
 import numpy.ma as ma
 
 
-__version__ = '0.6.0'
+__version__ = '0.7.0'
+
+
+_SCALAR_KEY_TYPES = (int, np.integer)
+_KEY_TYPES = _SCALAR_KEY_TYPES + (slice, tuple, np.ndarray)
+
+
+def _is_scalar(key):
+    return isinstance(key, (int, np.integer))
 
 
 class AxisSupportError(StandardError):
@@ -525,8 +534,7 @@ class Array(object):
         if len(keys) > self.ndim:
             raise IndexError('too many keys')
         for key in keys:
-            if not(isinstance(key, (int, np.integer,
-                                    slice, tuple, np.ndarray))):
+            if not(isinstance(key, _KEY_TYPES)):
                 raise TypeError('invalid index: {!r}'.format(key))
         return keys
 
@@ -673,13 +681,19 @@ class _ArrayAdapter(Array):
         Raises IndexError/TypeError for invalid keys.
 
         """
-        if isinstance(key, (int, np.integer)):
+        if _is_scalar(key):
             if key >= size or key < -size:
                 msg = 'index {0} is out of bounds for axis {1} with' \
                       ' size {2}'.format(key, axis, size)
                 raise IndexError(msg)
         elif isinstance(key, slice):
             pass
+        elif isinstance(key, np.ndarray) and key.dtype == np.dtype('bool'):
+            if key.size > size:
+                msg = 'too many boolean indices. Boolean index array ' \
+                      'of size {0} is greater than axis {1} with ' \
+                      'size {2}'.format(key.size, axis, size)
+                raise IndexError(msg)
         elif isinstance(key, collections.Iterable) and \
                 not isinstance(key, basestring):
             # Make sure we capture the values in case we've
@@ -703,7 +717,7 @@ class _ArrayAdapter(Array):
 
         """
         size = len(indices)
-        if isinstance(new_key, (int, np.integer)):
+        if _is_scalar(new_key):
             if new_key >= size or new_key < -size:
                 msg = 'index {0} is out of bounds for axis {1}' \
                       ' with size {2}'.format(new_key, axis, size)
@@ -711,6 +725,15 @@ class _ArrayAdapter(Array):
             result_key = indices[new_key]
         elif isinstance(new_key, slice):
             result_key = indices.__getitem__(new_key)
+        elif isinstance(new_key, np.ndarray) and \
+                new_key.dtype == np.dtype('bool'):
+            # Numpy boolean indexing.
+            if new_key.size > size:
+                msg = 'too many boolean indices. Boolean index array ' \
+                      'of size {0} is greater than axis {1} with ' \
+                      'size {2}'.format(new_key.size, axis, size)
+                raise IndexError(msg)
+            result_key = tuple(np.array(indices)[new_key])
         elif isinstance(new_key, collections.Iterable) and \
                 not isinstance(new_key, basestring):
             # Make sure we capture the values in case we've
@@ -740,7 +763,7 @@ class _ArrayAdapter(Array):
         while src_keys and new_keys:
             src_size = shape.pop(0)
             src_key = src_keys.pop(0)
-            if isinstance(src_key, (int, np.integer)):
+            if _is_scalar(src_key):
                 # An integer src_key means this dimension has
                 # already been sliced away - it's not visible to
                 # the new keys.
@@ -837,8 +860,7 @@ class NumpyArrayAdapter(_ArrayAdapter):
                 for i, key in tuple_keys:
                     cut_keys[i] = slice(None)
                 array = self.concrete[tuple(cut_keys)]
-                is_scalar = [isinstance(key, (int, np.integer))
-                             for key in cut_keys]
+                is_scalar = map(_is_scalar, cut_keys)
                 dimensions -= np.cumsum(is_scalar)
             else:
                 # Use ellipsis indexing to ensure we have a real ndarray
@@ -977,7 +999,7 @@ class ArrayStack(Array):
     def __setitem__(self, keys, value):
         assert len(keys) == self._stack.ndim
         for key in keys:
-            assert isinstance(key, (int, np.integer))
+            assert _is_scalar(key)
         assert isinstance(value, Array), type(value)
         self._stack[keys] = value
 
@@ -1130,15 +1152,14 @@ class LinearMosaic(Array):
             # then it's safe to just pass the keys to each tile.
             tile = self._tiles[0]
             tiles = [tile[keys] for tile in self._tiles]
-            f = lambda key: isinstance(key, (int, np.integer))
-            scalar_keys = filter(f, keys)
+            scalar_keys = filter(_is_scalar, keys)
             result = LinearMosaic(tiles, axis - len(scalar_keys))
         else:
             axis_lengths = [tile.shape[axis] for tile in self._tiles]
             offsets = np.cumsum([0] + axis_lengths[:-1])
             splits = offsets - 1
             axis_key = keys[axis]
-            if isinstance(axis_key, (int, np.integer)):
+            if _is_scalar(axis_key):
                 # Find the single relevant tile
                 tile_index = np.searchsorted(splits, axis_key) - 1
                 tile = self._tiles[tile_index]
@@ -1364,6 +1385,102 @@ class _CountMaskedStreamsHandler(_AggregationStreamsHandler):
         self.running_count += np.ma.count(data, axis=self.axis)
 
 
+class _MinStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.result = np.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.result
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.result = np.min(data, axis=self.axis)
+
+
+class _MinMaskedStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.result = np.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.result
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.ma.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.result = np.min(data, axis=self.axis)
+
+
+class _MaxStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.result = np.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.result
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.result = np.max(data, axis=self.axis)
+
+
+class _MaxMaskedStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.result = np.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.result
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.ma.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.result = np.max(data, axis=self.axis)
+
+
+class _SumStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.running_total = np.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.running_total
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.running_total += np.sum(data, axis=self.axis)
+
+
+class _SumMaskedStreamsHandler(_AggregationStreamsHandler):
+    def bootstrap(self, shape):
+        self.running_total = np.ma.zeros(shape, dtype=self.array.dtype)
+
+    def finalise(self):
+        array = self.running_total
+        # Promote array-scalar to 0-dimensional array.
+        if array.ndim == 0:
+            array = np.ma.array(array)
+        chunk = Chunk(self.current_keys, array)
+        return chunk
+
+    def process_data(self, data):
+        self.running_total += np.sum(data, axis=self.axis)
+
+
 class _MeanStreamsHandler(_AggregationStreamsHandler):
     def __init__(self, array, axis, mdtol):
         # The mdtol argument is not applicable to non-masked arrays
@@ -1573,8 +1690,15 @@ class _Aggregation(ComputedArray):
 
     def __getitem__(self, keys):
         keys = self._normalise_keys(keys)
-        keys = (slice(None),) + keys
-        return _Aggregation(self._array[keys], self._axis,
+        # Insert an ':' into these keys to get keys for self._array.
+        keys = list(keys)
+        keys[self._axis:self._axis] = [slice(None)]
+        keys = tuple(keys)
+        # Reduce the aggregation-axis by the number of prior dimensions that
+        # get removed by the indexing operation.
+        scalar_axes = map(_is_scalar, keys[:self._axis])
+        result_axis = self._axis - __builtin__.sum(scalar_axes)
+        return _Aggregation(self._array[keys], result_axis,
                             self._streams_handler_class,
                             self._masked_streams_handler_class,
                             self.dtype,
@@ -1602,11 +1726,11 @@ def _normalise_axis(axis, array):
     # TypeError/ValueError.
     if axis is None:
         axes = None
-    elif isinstance(axis, (int, np.integer)):
+    elif _is_scalar(axis):
         axes = (axis,)
     elif (isinstance(axis, collections.Iterable) and
             not isinstance(axis, (basestring, collections.Mapping)) and
-            all(map(lambda x: isinstance(x, (int, np.integer)), axis))):
+            all(map(_is_scalar, axis))):
         axes = tuple(axis)
     else:
         raise TypeError('axis must be None, int, or iterable of ints')
@@ -1644,6 +1768,93 @@ def count(a, axis=None):
                         np.dtype('i'), {})
 
 
+def min(a, axis=None):
+    """
+    Request the minimum of an Array over any number of axes.
+
+    .. note:: Currently limited to operating on a single axis.
+
+    Parameters
+    ----------
+    a : Array object
+        The object whose minimum is to be found.
+    axis : None, or int, or iterable of ints
+        Axis or axes along which the operation is performed. The default
+        (axis=None) is to perform the operation over all the dimensions of the
+        input array. The axis may be negative, in which case it counts from
+        the last to the first axis. If axis is a tuple of ints, the operation
+        is performed over multiple axes.
+
+    Returns
+    -------
+    out : Array
+        The Array representing the requested mean.
+    """
+    axes = _normalise_axis(axis, a)
+    assert axes is not None and len(axes) == 1
+    return _Aggregation(a, axes[0],
+                        _MinStreamsHandler, _MinMaskedStreamsHandler,
+                        a.dtype, {})
+
+
+def max(a, axis=None):
+    """
+    Request the maximum of an Array over any number of axes.
+
+    .. note:: Currently limited to operating on a single axis.
+
+    Parameters
+    ----------
+    a : Array object
+        The object whose maximum is to be found.
+    axis : None, or int, or iterable of ints
+        Axis or axes along which the operation is performed. The default
+        (axis=None) is to perform the operation over all the dimensions of the
+        input array. The axis may be negative, in which case it counts from
+        the last to the first axis. If axis is a tuple of ints, the operation
+        is performed over multiple axes.
+
+    Returns
+    -------
+    out : Array
+        The Array representing the requested max.
+    """
+    axes = _normalise_axis(axis, a)
+    assert axes is not None and len(axes) == 1
+    return _Aggregation(a, axes[0],
+                        _MaxStreamsHandler, _MaxMaskedStreamsHandler,
+                        a.dtype, {})
+
+
+def sum(a, axis=None):
+    """
+    Request the sum of an Array over any number of axes.
+
+    .. note:: Currently limited to operating on a single axis.
+
+    Parameters
+    ----------
+    a : Array object
+        The object whose summation is to be found.
+    axis : None, or int, or iterable of ints
+        Axis or axes along which the operation is performed. The default
+        (axis=None) is to perform the operation over all the dimensions of the
+        input array. The axis may be negative, in which case it counts from
+        the last to the first axis. If axis is a tuple of ints, the operation
+        is performed over multiple axes.
+
+    Returns
+    -------
+    out : Array
+        The Array representing the requested sum.
+    """
+    axes = _normalise_axis(axis, a)
+    assert axes is not None and len(axes) == 1
+    return _Aggregation(a, axes[0],
+                        _SumStreamsHandler, _SumMaskedStreamsHandler,
+                        a.dtype, {})
+
+
 def mean(a, axis=None, mdtol=1):
     """
     Request the mean of an Array over any number of axes.
@@ -1845,11 +2056,14 @@ def _sliced_shape(shape, keys):
     sliced_shape = []
     # TODO: Watch out for more keys than shape entries.
     for size, key in itertools.izip_longest(shape, keys):
-        if isinstance(key, (int, np.integer)):
+        if _is_scalar(key):
             continue
         elif isinstance(key, slice):
             size = len(range(*key.indices(size)))
             sliced_shape.append(size)
+        elif isinstance(key, np.ndarray) and key.dtype == np.dtype('bool'):
+            # Numpy boolean indexing.
+            sliced_shape.append(__builtin__.sum(key))
         elif isinstance(key, (tuple, np.ndarray)):
             sliced_shape.append(len(key))
         else:
